<?php

namespace Drupal\Tests\acquia_perz\Kernel;

use Drupal\acquia_perz\Form\AdminSettingsForm;
use Drupal\Core\Form\FormInterface;
use Drupal\Core\Form\FormState;

/**
 * Tests Admin Settings Form.
 *
 * @coversDefaultClass \Drupal\acquia_perz\Form\AdminSettingsForm
 * @group acquia_perz
 */
class AdminSettingsFormTest extends PerzGraphQLTestBase {

  /**
   * The Admin Settings Form elements form object under test.
   *
   * @var \Drupal\acquia_perz\Form\AdminSettingsForm
   */
  protected $adminSettingsForm;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'acquia_contenthub',
    'acquia_contenthub_publisher',
    'acquia_lift',
    'acquia_lift_publisher',
    'depcalc',
  ];

  /**
   * {@inheritdoc}
   *
   * @covers ::\__construct()
   */
  protected function setUp(): void {
    parent::setUp();
    $this->installConfig('acquia_perz');
    $this->installConfig('acquia_lift');
    $this->installConfig('acquia_lift_publisher');
    $this->adminSettingsForm = AdminSettingsForm::create($this->container);
    $config = $this->config('acquia_lift_publisher.entity_config');
    $config->set('render_role', 'anonymous');
    $config->set('view_modes', $this->getMockAcquiaLiftViewModes());
    $config->save();

    $mock_acquia_lift_settings = $this->getMockAcquiaLiftSettings();
    $config = $this->config('acquia_lift.settings');
    $config->set('credential', $mock_acquia_lift_settings['credential']);
    $config->set('identity', $mock_acquia_lift_settings['identity']);
    $config->set('field_mappings', $mock_acquia_lift_settings['field_mappings']);
    $config->set('udf_person_mappings', $mock_acquia_lift_settings['udf_person_mappings']);
    $config->set('udf_touch_mappings', $mock_acquia_lift_settings['udf_touch_mappings']);
    $config->set('udf_event_mappings', $mock_acquia_lift_settings['udf_event_mappings']);
    $config->set('visibility', $mock_acquia_lift_settings['visibility']);
    $config->set('advanced', $mock_acquia_lift_settings['advanced']);
    $config->set('langcode', $mock_acquia_lift_settings['langcode']);
    $config->save();

  }

  /**
   * Test Admin Settings Form.
   */
  public function testAdminSettingsForm() {
    $this->assertInstanceOf(FormInterface::class, $this->adminSettingsForm);
    $id = $this->adminSettingsForm->getFormId();
    $this->assertEquals('acquia_perz_settings_form', $id);
  }

  /**
   * Test Export Form Enqueue Content.
   */
  public function testConfigurationMigration() {
    $triggering_element = [
      '#name' => 'migrate_configuration',
      "#type" => "submit",
    ];

    $form_state = (new FormState())->setTriggeringElement($triggering_element);
    $form = $this->adminSettingsForm->buildForm([], $form_state);
    $this->adminSettingsForm->submitForm($form, $form_state);

    // Test entity configuration migration.
    $migrated_view_modes = $this->config('acquia_perz.entity_config')->get('view_modes');
    $expected_view_modes = $this->getAcquiaPerzExpectedViewModes();
    $this->assertEqualsCanonicalizing($expected_view_modes, $migrated_view_modes);
    $migrated_settings = $this->config('acquia_perz.settings');

    // Test the settings migration.
    $expected_settings = $this->getMockAcquiaLiftSettings();
    $this->assertEqualsCanonicalizing($expected_settings['identity'], $migrated_settings->get('identity'));
    $this->assertEqualsCanonicalizing($expected_settings['field_mappings'], $migrated_settings->get('field_mappings'));
    $this->assertEqualsCanonicalizing($expected_settings['udf_person_mappings'], $migrated_settings->get('udf_person_mappings'));
    $this->assertEqualsCanonicalizing($expected_settings['udf_touch_mappings'], $migrated_settings->get('udf_touch_mappings'));
    $this->assertEqualsCanonicalizing($expected_settings['udf_event_mappings'], $migrated_settings->get('udf_event_mappings'));
    $this->assertEqualsCanonicalizing($expected_settings['visibility'], $migrated_settings->get('visibility'));
    $this->assertEquals($expected_settings['advanced']['bootstrap_mode'], $migrated_settings->get('advanced.bootstrap_mode'));
    $this->assertEquals($expected_settings['advanced']['content_replacement_mode'], $migrated_settings->get('advanced.content_replacement_mode'));
    $this->assertEquals($expected_settings['advanced']['content_origins'], $migrated_settings->get('advanced.content_origins'));
    $this->assertEquals($expected_settings['langcode'], $migrated_settings->get('langcode'));

  }

  /**
   * Mock Acquia Lift View Modes.
   *
   * @return array
   *   Returns mock acquia lift view modes.
   */
  public function getMockAcquiaLiftViewModes() {
    return [
      'block_content' => [
        'basic' => [
          'default' => 1,
          'full' => 1,
        ],
      ],
      'node' => [
        'article' => [
          'default' => 1,
          'full' => 1,
          'acquia_lift_preview_image' => 'field_image',
          'rss' => 1,
          'teaser' => 1,
        ],
        'page' => [
          'default' => 1,
          'full' => 1,
          'rss' => 1,
          'teaser' => 1,
        ],
      ],
      'taxonomy_term' => [
        'tags' => [
          'default' => 1,
        ],
      ],
    ];
  }

  /**
   * Mock Expected Acquia Perz View Modes.
   *
   * @return array
   *   Returns expected acquia perz view modes.
   */
  public function getAcquiaPerzExpectedViewModes() {
    return [
      'block_content' => [
        'basic' => [
          'default' => [
            'render_role' => 'anonymous',
          ],
          'full' => [
            'render_role' => 'anonymous',
          ],
        ],
      ],
      'node' => [
        'article' => [
          'default' => [
            'render_role' => 'anonymous',
            'preview_image' => 'field_image',
          ],
          'full' => [
            'render_role' => 'anonymous',
            'preview_image' => 'field_image',
          ],
          'rss' => [
            'render_role' => 'anonymous',
            'preview_image' => 'field_image',
          ],
          'teaser' => [
            'render_role' => 'anonymous',
            'preview_image' => 'field_image',
          ],
        ],
        'page' => [
          'default' => [
            'render_role' => 'anonymous',
          ],
          'full' => [
            'render_role' => 'anonymous',
          ],
          'rss' => [
            'render_role' => 'anonymous',
          ],
          'teaser' => [
            'render_role' => 'anonymous',
          ],
        ],
      ],
      'taxonomy_term' => [
        'tags' => [
          'default' => [
            'render_role' => 'anonymous',
          ],
        ],
      ],
    ];
  }

  /**
   * Mock Acquia Lift Settings.
   *
   * @return array
   *   Returns mock acquia lift Settings.
   */
  public function getMockAcquiaLiftSettings() {
    return [
      'credential' => [
        'account_id' => 'TEST123',
        'site_id' => 'abc123',
        'assets_url' => 'abc',
        'decision_api_url' => 'def',
      ],
      'identity' => [
        'capture_identity' => FALSE,
        'identity_parameter' => 'Test',
        'identity_type_parameter' => '',
        'default_identity_type' => '',
      ],
      'field_mappings' => [
        'content_section' => 'field_tags',
        'content_keywords' => 'field_tags',
        'persona' => 'field_tags',
      ],
      'udf_person_mappings' => [
        'person_udf1' => [
          'id' => 'person_udf1',
          'value' => 'field_tags',
          'type' => 'taxonomy',
        ],
      ],
      'udf_touch_mappings' => [
        'touch_udf1' => [
          'id' => 'touch_udf1',
          'value' => 'field_tags',
          'type' => 'taxonomy',
        ],
      ],
      'udf_event_mappings' => [
        'event_udf1' => [
          'id' => 'event_udf1',
          'value' => 'field_tags',
          'type' => 'taxonomy',
        ],
      ],
      'visibility' => [
        'path_patterns' => '/test',
      ],
      'advanced' => [
        'bootstrap_mode' => 'auto',
        'content_replacement_mode' => 'trusted',
        'cdf_version' => 2,
        'content_origins' => 'asfsadf',
      ],
      'langcode' => 'en',
    ];
  }

}
