<?php

namespace Drupal\Tests\acquia_perz\Kernel;

use Drupal\Tests\acquia_perz\Traits\CustomParagraphsTestTrait;
use Drupal\Tests\paragraphs\FunctionalJavascript\ParagraphsTestBaseTrait;

/**
 * Tests discovery endpoint.
 *
 * @group acquia_perz
 */
class DiscoveryEndpointTest extends PerzGraphQLTestBase {

  use CustomParagraphsTestTrait;
  use ParagraphsTestBaseTrait;

  /**
   * {@inheritdoc}
   */
  protected $graphQLQuery = <<<GQL
  query discoverEntities(\$page: Int! = 0) {
    discover_entities(page: \$page) {
      page_info {
        total_count
        current_page_count
        current_page
        next_page
        prev_page
      }
      items {
        entity_type_id
        entity_uuid
      }
    }
  }
GQL;

  /**
   * {@inheritdoc}
   */
  protected $graphqlServer = NULL;

  /**
   * {@inheritdoc}
   */
  protected $viewModeDefaultValue = [
    'render_role' => 'anonymous',
    'preview_image' => '',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->graphqlServer = $this->createGraphqlServer(
      'Perz',
      'perz',
      'export_entities',
      '/perz3'
    );
    // We need to see exception details in response.
    $this->graphqlServer->set('debug_flag', TRUE)->save();
    // Create current user and setup graphql permission.
    $this->setUpCurrentUser([], ["execute {$this->graphqlServer->id()} arbitrary graphql requests"]);
  }

  /**
   * Creates four nodes and ensures that they are loaded correctly.
   */
  public function testDiscoveryEndpoint() {
    $bundle = 'news';
    $this->drupalCreateContentType([
      'type' => $bundle,
      'name' => $bundle,
    ]);
    $paragraph_field_name = 'field_paragraph';
    $paragraph_type = 'header_test';

    $this->createParagraphType($paragraph_type);
    $this->addParagraphFieldInBundle(
      'node',
      $bundle,
      $paragraph_field_name,
      $paragraph_type
    );

    // Create 2 nodes + 2 paragraphs.
    $paragraph1 = $this->createParagraph(
      $paragraph_type,
      'Paragraph title1',
      TRUE
    );
    $node1 = $this->createNodeWithParagraph(
      $bundle,
      'News title1',
      'News body1',
      $paragraph_field_name,
      $paragraph1
    );
    $paragraph2 = $this->createParagraph(
      $paragraph_type,
      'Paragraph title2',
      TRUE
    );
    $node2 = $this->createNodeWithParagraph(
      $bundle,
      'News title2',
      'News body2',
      $paragraph_field_name,
      $paragraph2
    );

    // 4 terms.
    $vocabulary = $this->createVocabulary();
    $term1 = $this->createTerm($vocabulary);
    $term2 = $this->createTerm($vocabulary);
    $term3 = $this->createTerm($vocabulary);
    $term4 = $this->createTerm($vocabulary);

    // 3 custom blocks.
    $block_type = $this->createCustomBlockType();
    $custom_block1 = $this->createCustomBlock($block_type);
    $custom_block2 = $this->createCustomBlock($block_type);
    $custom_block3 = $this->createCustomBlock($block_type);

    $graphql_query_variables = [
      'page' => 0,
    ];

    // 1. Use case when no entities are selected on perz form -
    // empty result with exception message should be returned.
    $this->setUpPerzEntityTypes([]);
    $result = $this->sendQuery(
      $this->graphQLQuery,
      $this->graphqlServer,
      $graphql_query_variables
    );
    $this->assertEquals(200, $result->getStatusCode());
    $json_content = json_decode($result->getContent(), TRUE);
    $this->assertNull($json_content['data']['discover_entities']);
    if (isset($json_content['errors'])
      && isset($json_content['errors'][0])
      && isset($json_content['errors'][0]['debugMessage'])) {
      $this->assertEquals("No entity types are available for the export.",
        $json_content['errors'][0]['debugMessage']);
    }

    // 2. Use case when only nodes are selected on perz form. Expect 2 nodes
    // to be returned.
    $entity_types = [
      'node' => [
        $bundle => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $expected_entities = [
      'node' => [
        $node1->uuid(),
        $node2->uuid(),
      ],
    ];
    // Check for different page max sizes.
    $this->checkDiscoveryEndpoint(1, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(2, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(3, $entity_types, $expected_entities);

    // 3. Use case when only nodes and terms are selected on perz form -
    // expect 2 nodes and 4 terms to be returned. Also check that all
    // entities are unique.
    $entity_types = [
      'node' => [
        $bundle => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
      'taxonomy_term' => [
        $vocabulary->id() => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $expected_entities = [
      'node' => [
        $node1->uuid(),
        $node2->uuid(),
      ],
      'taxonomy_term' => [
        $term1->uuid(),
        $term2->uuid(),
        $term3->uuid(),
        $term4->uuid(),
      ],
    ];
    // Check for different page max sizes.
    $this->checkDiscoveryEndpoint(1, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(2, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(3, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(4, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(5, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(6, $entity_types, $expected_entities);

    // 4. Add 2 entity types - custom block and paragraph
    // - expect 11 entities.
    $entity_types = [
      'node' => [
        $bundle => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
      'taxonomy_term' => [
        $vocabulary->id() => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
      'block_content' => [
        $block_type->id() => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
      'paragraph' => [
        $paragraph_type => [
          'default' => $this->viewModeDefaultValue,
        ],
      ],
    ];
    $expected_entities = [
      'node' => [
        $node1->uuid(),
        $node2->uuid(),
      ],
      'taxonomy_term' => [
        $term1->uuid(),
        $term2->uuid(),
        $term3->uuid(),
        $term4->uuid(),
      ],
      'block_content' => [
        $custom_block1->uuid(),
        $custom_block2->uuid(),
        $custom_block3->uuid(),
      ],
      'paragraph' => [
        $paragraph1->uuid(),
        $paragraph2->uuid(),
      ],
    ];
    // Check for different page max sizes.
    $this->checkDiscoveryEndpoint(1, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(2, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(3, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(4, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(5, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(6, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(7, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(8, $entity_types, $expected_entities);
    $this->checkDiscoveryEndpoint(9, $entity_types, $expected_entities);
  }

}
