<?php

namespace Drupal\Tests\acquia_perz\Kernel;

use Drupal\acquia_connector\Subscription;
use Drupal\graphql\Entity\Server;
use Drupal\graphql\Entity\ServerInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\acquia_perz\Traits\CustomBlockTestTrait;
use Drupal\Tests\node\Traits\ContentTypeCreationTrait;
use Drupal\Tests\node\Traits\NodeCreationTrait;
use Drupal\Tests\taxonomy\Traits\TaxonomyTestTrait;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * {@inheritdoc}
 */
abstract class PerzGraphQLTestBase extends KernelTestBase {

  use NodeCreationTrait {
    getNodeByTitle as drupalGetNodeByTitle;
    createNode as drupalCreateNode;
  }
  use UserCreationTrait {
    createUser as drupalCreateUser;
    createRole as drupalCreateRole;
    createAdminRole as drupalCreateAdminRole;
  }
  use ContentTypeCreationTrait {
    createContentType as drupalCreateContentType;
  }

  use TaxonomyTestTrait;

  use CustomBlockTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'paragraphs',
    'entity_reference_revisions',
    'file',
    'node',
    'datetime',
    'user',
    'system',
    'block',
    'block_content',
    'filter',
    'field',
    'text',
    'language',
    'locale',
    'content_translation',
    'taxonomy',
    'typed_data',
    'graphql',
    'image',
    'acquia_perz',
    'acquia_connector',
    'path_alias',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $site_base_url = \Drupal::service('request_stack')->getCurrentRequest()->getSchemeAndHttpHost();
    $subscriptionServiceMock = $this->createMock(Subscription::class);
    $subscriptionServiceMock->expects($this->any())
      ->method('getSubscription')
      ->willReturn([
        'acquia_perz' => [
          'api_key' => 'AUTH-TEST-1',
          'secret_key' => 'a491206bc0a61d51e4dfac8a81d5d1a7',
          'account_id' => 'PERZTESTv3',
          'endpoint' => $site_base_url,
        ],
      ]
      );
    $container = \Drupal::getContainer();
    $container->set('acquia_connector.subscription', $subscriptionServiceMock);

    $this->installSchema('system', 'sequences');
    $this->installSchema('node', 'node_access');
    $this->installSchema('user', ['users_data']);
    $this->installSchema('locale', ['locales_source', 'locales_target', 'locales_location', 'locale_file']);
    $this->installConfig('filter');
    $this->installConfig('node');
    $this->installConfig('graphql');
    $this->installConfig('image');
    $this->installConfig('acquia_perz');
    $this->installEntitySchema('node');
    $this->installEntitySchema('user');
    $this->installEntitySchema('block_content');
    $this->installEntitySchema('taxonomy_term');
    $this->installEntitySchema('graphql_server');
    $this->installEntitySchema('date_format');
    $this->installEntitySchema('paragraph');
    \Drupal::moduleHandler()->loadInclude('paragraphs', 'install');
    // Setup environment.
    \Drupal::service('state')->set('acquia_perz.environment', 'test_dev');
  }

  /**
   * Get remote entities from current graphQL endpoint and compare.
   *
   * Results with expect entities.
   *
   * @param int $page_size
   *   The page size (pager).
   * @param array $view_modes
   *   List of entity types > bundles > view modes.
   * @param array $expected_entities
   *   Expected entities (uuids).
   */
  public function checkDiscoveryEndpoint($page_size, array $view_modes, array $expected_entities) {
    if (!($this->graphqlServer instanceof Server)) {
      throw new \LogicException('GraphQL server is not set.');
    }
    $this->setUpPerzEntityTypes($view_modes);
    $this->setUpPerzPageSize($page_size);
    $actual_entities = $this->getRemoteEntitiesByTypes(
      array_keys($view_modes)
    );

    foreach ($actual_entities as $actual_entity_type_id => $actual_uuids) {
      $expected_uuids = $expected_entities[$actual_entity_type_id];
      $this->assertCount(count($actual_uuids), $expected_uuids);
      foreach ($expected_uuids as $expected_uuid) {
        $this->assertContains($expected_uuid, $actual_uuids);
      }
      unset($expected_entities[$actual_entity_type_id]);
    }
    $this->assertEmpty($expected_entities, "Some entities are absent.");
  }

  /**
   * Get remote entities by entity types.
   *
   * @param array $entity_types
   *   List of entity types that should be categorized with uuids.
   *
   * @return array
   *   List of entity types > entity uuids.
   */
  protected function getRemoteEntitiesByTypes(array $entity_types) {
    $result = [];
    $graphql_query_body = $this->graphQLQuery;
    $graphql_query_variables = [
      'page' => 0,
    ];
    do {
      $response = $this->sendQuery(
        $graphql_query_body,
        $this->graphqlServer,
        $graphql_query_variables
      );
      $json_content = json_decode($response->getContent(), TRUE);
      $page_info = $json_content['data']['discover_entities']['page_info'];
      $items = $json_content['data']['discover_entities']['items'];
      foreach ($items as $item) {
        if (in_array($item['entity_type_id'], $entity_types)) {
          if (isset($result[$item['entity_type_id']])
            && !in_array($item['entity_uuid'], $result[$item['entity_type_id']])) {
            $result[$item['entity_type_id']][] = $item['entity_uuid'];
          }
          else {
            $result[$item['entity_type_id']] = [$item['entity_uuid']];
          }
        }
      }
    } while ($graphql_query_variables['page'] = $page_info['next_page']);
    return $result;
  }

  /**
   * Create and return graphql server entity.
   *
   * @param string $label
   *   The graphql server machine name.
   * @param string $name
   *   The graphql server title.
   * @param string $schema
   *   The graphql server schema.
   * @param string $endpoint
   *   The graphql server endpoint.
   *
   * @return \Drupal\Core\Entity\EntityBase|\Drupal\Core\Entity\EntityInterface|\Drupal\graphql\Entity\Server
   *   Created graphql server entity.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function createGraphqlServer($label, $name, $schema, $endpoint) {
    $graphql_server = Server::create();
    $graphql_server->set('label', $label);
    $graphql_server->set('name', $name);
    $graphql_server->set('schema', $schema);
    $graphql_server->set('endpoint', $endpoint);
    $graphql_server->save();
    return $graphql_server;
  }

  /**
   * Setup Perz configuration config.
   *
   * @param array $view_modes
   *   The array of entity_types > bundles > view_modes.
   */
  protected function setUpPerzEntityTypes(array $view_modes) {
    $entity_settings = $this->config('acquia_perz.entity_config');
    $entity_settings->set('view_modes', $view_modes);
    $entity_settings->save();
  }

  /**
   * Setup Perz page size config.
   *
   * @param int $page_size
   *   The page max size.
   */
  protected function setUpPerzPageSize($page_size) {
    $cis_settings = $this->config('acquia_perz.settings');
    $cis_settings->set('api.discovery_endpoint_page_size', $page_size);
    $cis_settings->save();
  }

  /**
   * Sends graphql query and returns response.
   *
   * @param string $query
   *   The graphql query.
   * @param \Drupal\graphql\Entity\ServerInterface $server
   *   The graphql server.
   * @param array $variables
   *   The graphql variables.
   *
   * @return mixed
   *   The graphql response.
   *
   * @throws \Exception
   */
  protected function sendQuery($query, ServerInterface $server, array $variables = []) {
    if (!($server instanceof Server)) {
      throw new \LogicException('Invalid server.');
    }

    $data = [
      'query' => $query,
      'variables' => $variables,
    ];
    $client_factory = $this->container->get('acquia_perz.client_factory');
    $result = $client_factory->sendQuery($data);
    // var_dump($result);
    return $result;
  }

}
